﻿using UnityEngine;
using System.Collections.Generic;
using UnityEngine.EventSystems;

public class CentralBox : MonoBehaviour {

	public GameObject printPaperPrefab;
	public GameObject printSpawn;

	public List<Pipe> pipeEnds = new List<Pipe>();

	public List<ScheduledStream> streams = new List<ScheduledStream>();

	public GameObject scheduledSteamPrefab;

	public GameObject help1;

	public GameObject reward1;

	public Blackout blackout;
	public UnityEngine.UI.Text endText;

	public void Start()
	{
		foreach (var p in this.GetComponentsInChildren<Pipe>())
		{
			this.pipeEnds.Add(p);
		}
	}

	public void Update()
	{
		if (Clock.ticking)
		{
			if (Clock.totalTime > 130 && !this.help1.activeSelf)
				this.help1.SetActive(true);

			if (this.streams.Count == 0)
			{
				this.CreateStream(Clock.totalTime < 60 ? 0 : 1);
			}
			else if (this.streams.Count == 1 && Clock.totalTime > 60)
			{
				this.CreateStream(Clock.totalTime < 160 ? 0 : 1);
			}

			if (Clock.totalTime > 5 * 60)
			{
				Clock.ticking = false;
				this.blackout.text.enabled = false;
				this.endText.gameObject.SetActive(true);
				this.blackout.text = this.endText;
				this.blackout.blackout = 1;
			}
		}
	}

	public Pipe GetRandomNotUsedPipe(ScheduledStream stream, int rangeStart, int rangeEnd)
	{
		Pipe pipe = null;
		var i = 0;
		while (i++ < 50 && pipe == null)
		{
			var nr = Random.Range(rangeStart, rangeEnd);
			pipe = this.pipeEnds[nr];

			foreach (var s in this.streams)
			{
				if (s.startTime > stream.endTime)
					continue;
				if (s.endTime < stream.startTime)
					continue;
				if (s.endPipe == pipe || s.startPipe == pipe)
				{
					pipe = null;
					break;
				}
			}
		}
		return pipe;
	}

	public ScheduledStream CreateStream(int diff = 0)
	{
		var stream = (GameObject.Instantiate(this.scheduledSteamPrefab) as GameObject).GetComponent<ScheduledStream>();
		stream.transform.parent = this.transform;

		stream.startTime = Clock.totalTime + Random.Range(20, 60);
		//stream.startTime = Clock.totalTime;
		stream.endTime = stream.startTime + Random.Range(30, 120);

		this.streams.Add(stream);
		stream.centralBox = this;

		if (diff == 0)
		{
			var side = Random.Range(0,2);
			stream.startPipe = GetRandomNotUsedPipe(stream, side * 5, (side + 1) * 5);
			stream.endPipe = GetRandomNotUsedPipe(stream, side * 5, (side + 1) * 5);
		}
		else if (diff == 1)
		{
			stream.startPipe = GetRandomNotUsedPipe(stream, 0, this.pipeEnds.Count);
			stream.endPipe = GetRandomNotUsedPipe(stream, 0, this.pipeEnds.Count);
		}

		var print = (GameObject.Instantiate(this.printPaperPrefab) as GameObject).GetComponent<Paper>();
		print.transform.parent = this.printSpawn.transform;
		print.transform.localPosition = Vector3.zero;
		print.contents.text = "";
		print.contents.text += "OUTPT: "+ stream.startPipe.label.text +"\n";
		print.contents.text += "INPUT: "+ stream.endPipe.label.text +"\n";
		print.contents.text += "START: "+ Clock.FormatTime(stream.startTime) +"\n";
		print.contents.text += "END  : "+ Clock.FormatTime(stream.endTime) +"\n";
		print.contents.fontSize = 20;

		stream.paper = print;

		return stream;
	}

	public void ReleaseReward()
	{
		if (!this.reward1.activeSelf)
			this.reward1.SetActive(true);
	}
}

