﻿using UnityEngine;
using UnityEngine.EventSystems;
using System.Collections;
using System.Collections.Generic;

public class Hammer : MonoBehaviour, IPointerUpHandler, IPointerDownHandler
{
	public HammerHead head;
	public List<Pipe> underHead = new List<Pipe>();

	public AudioClip hitWood;
	public AudioClip hitPipe;
	public AudioClip hitPipeEcho;

	public bool raising = false;
	public float maxRaise = 0f;

	public void OnPointerDown(PointerEventData eventData)
	{
		if (eventData.pointerId == -1)
		{
			this.raising = true;
		}
	}
	
	public void OnPointerUp(PointerEventData eventData)
	{
		if (eventData.pointerId == -1)
		{
			this.raising = false;
		}
	}

	public void Update()
	{
		if (!this.raising && this.transform.localScale != Vector3.one)
		{
			this.transform.localScale -= 5f * Vector3.one * Time.deltaTime;

			if (this.transform.localScale.x <= 1)
			{
				this.transform.localScale = Vector3.one;

				var hits = Physics2D.CircleCastAll((Vector2)this.head.transform.position, 0.6f, Vector2.zero);

				var hitWood = true;
				var hitPipe = false;
				var hitPipeEcho = false;

				foreach (var hit in hits)
				{
					var p = hit.collider.GetComponent<Pipe>();
					if (p == null || p.transform == this.transform)
						continue;

					if (this.maxRaise > 1.25f)
					{
						if (p.GetComponent<Clock>())
							p.GetComponent<Clock>().broken += 1;

						if (p.GetComponent<RequestPipe>())
							StartCoroutine(p.GetComponent<RequestPipe>().DoClick());

						foreach (var c in p.connectors)
						{
							if (c.connectedTo != null)
								c.connectedTo.connectedTo = null;
							c.connectedTo = null;
						}
						p.SetParentPipe(p);
					}
					hitPipe = true;
					var parent = p.GetParentPipe();
					if (!parent.rigidbody2D.isKinematic && parent.canDrag)
					{
						parent.rigidbody2D.AddForce((head.transform.position - p.transform.position).normalized * -5000 * ((this.maxRaise - 1f)));
						parent.rigidbody2D.AddTorque(Random.Range(-1000, 1000) * ((this.maxRaise - 1f)));
					}
					else
						hitPipeEcho = true;

				}

				if (hitPipeEcho)
					AudioSource.PlayClipAtPoint(this.hitPipeEcho, Camera.main.transform.position, 0.25f);
				else if (hitPipe)
					AudioSource.PlayClipAtPoint(this.hitPipe, Camera.main.transform.position, 0.25f);
				else if (hitWood)
					AudioSource.PlayClipAtPoint(this.hitWood, Camera.main.transform.position, 0.25f);

			}
		}
		if (this.raising && this.transform.localScale.x < 2)
		{
			this.transform.localScale += 0.5f * Vector3.one * Time.deltaTime;
			this.maxRaise = this.transform.localScale.x;
		}
	}
}
