﻿using UnityEngine;
using UnityEngine.EventSystems;
using System.Collections.Generic;

public class Pipe : MonoBehaviour, IScrollHandler, IPointerClickHandler, IDragHandler, IEndDragHandler, IBeginDragHandler, IPointerDownHandler, IPointerUpHandler
{
	public HashSet<PipeConnector> connectors = new HashSet<PipeConnector>();

	public Pipe parentPipe;
	public HashSet<Pipe> childPipes = new HashSet<Pipe>();

	public AudioClip clickAudio;

	public bool isDragging = false;
	public bool isRotating = false;
	public Vector2 dragOffset = Vector2.zero;

	public PointerEventData rotateAround = null;
	public Vector2 dragDiff = Vector2.zero;
	public Vector2 prevDrag = Vector2.zero;

	public UnityEngine.UI.Text label;

	public bool canRotate = true;
	public bool canDrag = true;

	public bool rotateCC = false;

	public float gravity = 0.005f;

	public void SetParentPipe(Pipe pipe)
	{
		if (this.parentPipe == null)
			this.parentPipe = this;

		if (pipe == this)
		{
			if (this.parentPipe != this)
				this.parentPipe.childPipes.Remove(this);
			this.parentPipe = this;
			this.transform.parent = GameObject.Find("Pipes").transform;
			if (!this.gameObject.rigidbody2D)
			{
				this.gameObject.AddComponent<Rigidbody2D>();
				this.gameObject.rigidbody2D.gravityScale = 0;
				this.gameObject.rigidbody2D.mass = 10;
				this.gameObject.rigidbody2D.drag = 7;
				this.gameObject.rigidbody2D.angularDrag = 1;
				this.gameObject.rigidbody2D.gravityScale = this.gravity;
				this.gameObject.rigidbody2D.interpolation = RigidbodyInterpolation2D.Interpolate;
				this.gameObject.rigidbody2D.collisionDetectionMode = CollisionDetectionMode2D.Continuous;
			}
		}
		else
		{
			if (this.parentPipe != this)
			{
				if (this.parentPipe.GetAllChildPipes().Contains(pipe))
					Debug.Log("ho ho ho circularrr!");
				else
					this.parentPipe.SetParentPipe(pipe);
			}
			else
			{
				this.isDragging = false;
				Destroy(this.GetParentPipe().rigidbody2D);
				this.parentPipe = pipe.GetParentPipe();
				this.parentPipe.childPipes.Add(this);
				this.transform.parent = this.parentPipe.transform;
			}
		}
	}

	public HashSet<Pipe> GetAllChildPipes()
	{
		var all = new HashSet<Pipe>(this.childPipes);

		foreach (var c in this.childPipes)
			if (c != this)
				all.UnionWith(c.GetAllChildPipes());

		return all;
	}

	public Pipe GetParentPipe()
	{
		if (this.parentPipe != this && this.parentPipe != null)
			return this.parentPipe.GetParentPipe();
		else
			return this;
	}

	public void Start()
	{
		if (this.parentPipe == null)
		{
			this.parentPipe = this;
			foreach (var c in this.GetComponentsInChildren<PipeConnector>())
				c.Start();
		}
	}

	public void OnBeginDrag(PointerEventData data)
	{
		if (!this.canDrag)
			return;

		if (this.rigidbody2D == null)
		{
			this.GetParentPipe().OnBeginDrag(data);
		}
		else
		{
			this.rigidbody2D.gravityScale = 0;
			this.rigidbody2D.velocity = Vector2.zero;
			this.rigidbody2D.angularVelocity = 0;
			this.isDragging = true;
			this.dragOffset = data.position - (Vector2)Camera.main.WorldToScreenPoint(this.transform.position);
			this.prevDrag = data.position;
			this.dragDiff = Vector2.zero;
		}
	}

	public void OnEndDrag(PointerEventData data)
	{
		if (!this.canDrag)
			return;

		if (this.rigidbody2D == null)
		{
			this.GetParentPipe().OnEndDrag(data);
		}
		else
		{
			this.rigidbody2D.gravityScale = this.gravity;

			this.isDragging = false;

			if (this.dragDiff != Vector2.zero)
			{
				if (this.dragDiff.magnitude > 80)
					this.dragDiff = this.dragDiff.normalized * 80;
				this.rigidbody2D.AddForce(-this.dragDiff * 40 * this.rigidbody2D.mass);
			}
		}
	}

	public void OnDrag(PointerEventData data)
	{
		if (!this.canDrag)
			return;

		if (this.rigidbody2D == null)
		{
			this.GetParentPipe().OnDrag(data);
		}
		else if (this.rigidbody2D != null && !this.rigidbody2D.isKinematic)
		{
			var offset = (data.position - this.dragOffset);
			var point = Camera.main.ScreenToWorldPoint(new Vector3(offset.x, offset.y, 10));
			gameObject.transform.position = point;
			this.dragDiff = this.prevDrag - data.position;
			this.prevDrag = data.position;
		}
	}

	public void OnPointerClick(PointerEventData eventData)
	{
		if (eventData.pointerId == -2)
		{
			//this.GetParentPipe().transform.RotateAround(Camera.main.ScreenToWorldPoint(new Vector3(eventData.position.x, eventData.position.y, 10)), Vector3.forward, -90);
		}
	}

	public void Update()
	{
		if (this.rotateAround != null && this.canRotate && !this.GetParentPipe().rigidbody2D.isKinematic)
		{
			this.GetParentPipe().transform.RotateAround(
				Camera.main.ScreenToWorldPoint(new Vector3(this.rotateAround.position.x, this.rotateAround.position.y, 10)), 
				Vector3.forward, 
				-180 * Time.deltaTime * (rotateCC ? 1 : -1)
			);
		}
	}

	public void OnPointerDown(PointerEventData eventData)
	{
		var paper = this.GetComponent<Paper>();
		if (paper != null && paper.canvas.sortingOrder != Paper.maxPaperLevel)
		{
			Paper.maxPaperLevel++;
			paper.canvas.sortingOrder = Paper.maxPaperLevel;
		}
		


		if (eventData.pointerId == -2)
		{
			this.rotateAround = eventData;
			this.rotateCC = !this.rotateCC;
			this.isRotating = true;
			//this.transform.eulerAngles += new Vector3(0, 
		}
	}

	public void OnPointerUp(PointerEventData eventData)
	{
		if (eventData.pointerId == -2)
		{
			this.isRotating = false;
			this.rotateAround = null;
		}
	}

	public void OnScroll(PointerEventData eventData)
	{
		Debug.Log(eventData.scrollDelta);
	}
}
