﻿using UnityEngine;
using System.Collections;

public class PipeConnector : MonoBehaviour {

	public Pipe pipe;
	public bool isFemale = false;

	public PipeConnector connectedTo;

	public Pipe touches;

	public void Start() 
	{
		if (this.pipe != null)
			this.pipe.connectors.Add(this);
	}

	public PipeConnector GetEnd()
	{
		if (this.connectedTo != null)
			return this.connectedTo.GetOtherEnd();
		else
			return this;
	}
	
	public PipeConnector GetOtherEnd()
	{
		foreach (var c in this.pipe.connectors)
		{
			if (c != this)
			{
				if (c.connectedTo != null)
					return c.connectedTo.GetOtherEnd();
				else
					return c;
			}
		}
		return this;
	}

	public void ConnectWith(PipeConnector c, bool isParent = false)
	{
		if (c.isFemale == this.isFemale)
			return;

		c.connectedTo = this;
		this.connectedTo = c;
		
		var thisIsParent = false;
		if (this.pipe.GetParentPipe().rigidbody2D.isKinematic)
			thisIsParent = true;
		else if (c.pipe.GetParentPipe().rigidbody2D.isKinematic)
			thisIsParent = false;
		else 
			thisIsParent = isParent;
		
		if (thisIsParent)
		{
			var oldParent = c.pipe.GetParentPipe();
			c.pipe.SetParentPipe(this.pipe);
			
			var angleDiff = this.SignedAngleDiff(this.transform.eulerAngles.z, c.transform.eulerAngles.z);
			oldParent.transform.RotateAround(c.transform.position, Vector3.forward, angleDiff);
			
			var oldParentDiff = (oldParent.transform.position - c.pipe.transform.position);
			
			if (c.isFemale)
				oldParent.transform.position = this.transform.position + oldParentDiff;
			else
				oldParent.transform.position = this.pipe.transform.position + oldParentDiff + (c.pipe.transform.position - c.transform.position);
		}
		else
		{
			var oldParent = this.pipe.GetParentPipe();
			this.pipe.SetParentPipe(c.pipe);
			
			var angleDiff = this.SignedAngleDiff(c.transform.eulerAngles.z, this.transform.eulerAngles.z);
			oldParent.transform.RotateAround(this.transform.position, Vector3.forward, angleDiff);
			
			var oldParentDiff = (oldParent.transform.position - this.pipe.transform.position);
			
			if (this.isFemale)
				oldParent.transform.position = c.transform.position + oldParentDiff;
			else
				oldParent.transform.position = c.pipe.transform.position + oldParentDiff + (this.pipe.transform.position - this.transform.position);
		}		
	}
	
	
	void OnTriggerEnter2D(Collider2D other) 
	{
		var touches = other.gameObject.GetComponent<Pipe>();

		if (touches != null && this.connectedTo == null)
		{
			this.touches = touches;
			foreach (var c in this.touches.connectors)
			{
				var thisParent = this.pipe.GetParentPipe();
				var cParent = c.pipe.GetParentPipe();
				var isActive = (thisParent.isDragging || cParent.isDragging || thisParent.isRotating || cParent.isRotating);
				var isLastPiece = (thisParent.rigidbody2D.isKinematic && cParent.rigidbody2D.isKinematic);

				if (c.touches == this.pipe && c.isFemale != this.isFemale && (isActive || isLastPiece))
				{
					this.ConnectWith(c, thisParent.isDragging || thisParent.isRotating);

					AudioSource.PlayClipAtPoint(this.pipe.clickAudio, Camera.main.transform.position, 0.5f);

					return;
				}
			}
		}
	}

	float SignedAngle(Vector2 v1, Vector2 v2)
	{
		return Mathf.Atan2(v2.y, v2.x) - Mathf.Atan2(v1.y, v1.x);
	}

	float SignedAngleDiff(float angle1, float angle2)
	{
		return NormalizeAngle(angle1 - angle2 - 180);
	}

	float NormalizeAngle(float angle)
	{
		float newAngle = angle;
		while (newAngle <= -180) newAngle += 360;
		while (newAngle > 180) newAngle -= 360;
		return newAngle;
	}

	void OnTriggerExit2D(Collider2D other) 
	{
		this.touches = null;
	}
}
